﻿#region Copyright ©2007-2013, SIVECO Romania SA - All Rights Reserved
// ======================================================================
// Copyright ©2007-2013 SIVECO Romania SA - All Rights Reserved
// ======================================================================
// This file and its contents are protected by Romanian and International
// copyright laws. Unauthorized reproduction and/or distribution of all
// or any portion of the code contained herein is strictly prohibited
// and will result in severe civil and criminal penalties.
// Any violations of this copyright will be prosecuted
// to the fullest extent possible under law.
// ======================================================================
// THIS COPYRIGHT NOTICE MAY NOT BE REMOVED FROM THIS FILE.
// ======================================================================
#endregion

#region References
using System;
using System.Security.Cryptography.X509Certificates;
using System.Threading;
using System.Threading.Tasks;
using System.Windows.Forms;
using Cnas.Siui.TestConnectivity.Services;
using Cnas.Siui.TestConnectivity.Services.Parameters;
#endregion

namespace Cnas.Siui.TestConnectivity.Forms
{
    public partial class MainForm : Form
    {
        #region Constructors
        /// <summary>
        /// Creates a new instance of class <see cref="MainForm"/>
        /// </summary>
        public MainForm()
        {
            this.InitializeComponent();
        }
        #endregion

        #region Event handlers
        private void MainForm_Load( object sender, EventArgs e )
        {
            this.comboBoxCertificate.DataSource = Utils.X509Utils.GetCertificatesList();
        }

        private void MainForm_FormClosed( object sender, FormClosedEventArgs e )
        {
            Properties.Settings.Default.Save();
        }

        private void buttonCheckConnection_Click( object sender, System.EventArgs e )
        {
            this.TestConnection();
        }

        private void buttonGetPrescription_Click( object sender, System.EventArgs e )
        {
            this.GetPrescription();
        }

        private void buttonGetSeries_Click( object sender, System.EventArgs e )
        {
            this.GetPrescriptionSeries();
        }
        #endregion

        #region TestConnection methods
        private void TestConnection()
        {
            var certificate = this.comboBoxCertificate.SelectedItem as X509Certificate2;
            if( certificate == null )
            {
                MessageBox.Show( "Nu ati selectat cerficicatul!", this.Text, MessageBoxButtons.OK, MessageBoxIcon.Exclamation );
                return;
            }

            this.textBoxResult.Text = null;
            this.buttonCheckConnection.Enabled = false;
            this.buttonCheckConnection.Text = "Se verifica...";

            try
            {
                Task.Factory
                    .StartNew( () => new ConnectivityCheckerRemoteService( this.textBoxUrl.Text,
                                                                           this.textBoxUrlOcsp.Text,
                                                                           this.textBoxUsername.Text,
                                                                           this.textBoxPassword.Text,
                                                                           certificate,
                                                                           10000 ).GetResponse(),
                               TaskCreationOptions.LongRunning )
                    .ContinueWith( prev =>
                                   {
                                       this.buttonCheckConnection.Enabled = true;
                                       this.buttonCheckConnection.Text = "Verifica";
                                       if( prev.IsFaulted == false )
                                       {
                                           MessageBox.Show( prev.Result ? "Conexiunea s-a realizat cu succes." : "Conexiunea a esuat!",
                                                            this.Text,
                                                            MessageBoxButtons.OK,
                                                            prev.Result ? MessageBoxIcon.Information : MessageBoxIcon.Warning );
                                       }
                                       else
                                       {
                                           MessageBox.Show( string.Format( "Conexiunea a esuat:\n\n{0}", prev.Exception != null
                                                                                                             ? prev.Exception.GetBaseException().Message
                                                                                                             : "A aparut o eroare neasteptata." ),
                                                            this.Text, MessageBoxButtons.OK, MessageBoxIcon.Warning );
                                       }
                                   },
                                   CancellationToken.None, TaskContinuationOptions.ExecuteSynchronously, TaskScheduler.FromCurrentSynchronizationContext()
                    );
            }
            catch( AggregateException ex )
            {
                MessageBox.Show( ex.GetBaseException().Message, this.Text, MessageBoxButtons.OK, MessageBoxIcon.Warning );
                this.buttonCheckConnection.Enabled = true;
                this.buttonCheckConnection.Text = "Verifica";
            }
        }
        #endregion

        #region GetPrescription methods
        private void GetPrescription()
        {
            var certificate = this.comboBoxCertificate.SelectedItem as X509Certificate2;
            if( certificate == null )
            {
                MessageBox.Show( "Nu ati selectat cerficatul!", this.Text, MessageBoxButtons.OK, MessageBoxIcon.Warning );
                return;
            }

            this.textBoxResult.Text = null;
            this.buttonGetPrescription.Enabled = false;

            var service = new PrescriptionDownloaderWebService( this.textBoxPEUrlWs.Text, this.textBoxUsername.Text, this.textBoxPassword.Text, (X509Certificate2)this.comboBoxCertificate.SelectedItem, this.textBoxUrlOcsp.Text, 10000 );
            var parameters = new PrescriptionDownloaderParameters( this.textBoxPEProviderCode.Text, this.textBoxPEPhysicianStencilNo.Text, this.textBoxPEContractNo.Text, this.textBoxPEContractType.Text, this.textBoxPEOrgUnitCode.Text, this.textBoxPESeriesCode.Text, this.textBoxPESeriesNo.Text, this.dateTimePickerPEPrescriptionDate.Value );

            service.SendRequest( parameters, OnCompletedPrescription, OnFaultedPrescription );
        }

        private void OnFaultedPrescription( PrescriptionDownloaderParameters prescriptionDownloaderParameters, Exception exception )
        {
            this.buttonGetPrescription.Enabled = true;
            MessageBox.Show( string.Format( "Serviciul-Web a transmis urmatorul raspuns:\n\n{0}", exception.GetBaseException().Message ), this.Text, MessageBoxButtons.OK, MessageBoxIcon.Warning );
        }

        private void OnCompletedPrescription( PrescriptionDownloaderParameters prescriptionDownloaderParameters, string response )
        {
            this.buttonGetPrescription.Enabled = true;
            this.textBoxResult.Text = response;

            MessageBox.Show( string.IsNullOrEmpty( response ) ? "Reteta nu a fost preluata!" : "Reteta a fost preluata cu success!",
                             this.Text,
                             MessageBoxButtons.OK,
                             string.IsNullOrEmpty( response ) ? MessageBoxIcon.Warning : MessageBoxIcon.Information );
        }
        #endregion

        #region GetPrescriptionSeries methods
        private void GetPrescriptionSeries()
        {
            this.textBoxResult.Text = null;
            this.buttonGetSeries.Enabled = false;
            var service = new PrescriptionSeriesDownloaderWebService( this.textBoxSeriesUrl.Text, this.textBoxUsername.Text, this.textBoxPassword.Text, (X509Certificate2)this.comboBoxCertificate.SelectedItem, this.textBoxUrlOcsp.Text, 10000 );
            var parameters = new PrescriptionSeriesDownloaderParameters( this.textBoxSeriesCategoryCode.Text, this.textBoxSeriesOrgUnitCode.Text, this.textBoxSeriesUIC.Text, this.textBoxSeriesSubUnitCode.Text );
            service.SendRequest( parameters, OnCompletedSeries, OnFaultedSeries );
        }

        private void OnFaultedSeries( PrescriptionSeriesDownloaderParameters prescriptionDownloaderParameters, Exception exception )
        {
            this.buttonGetSeries.Enabled = true;
            MessageBox.Show( string.Format( "Serviciul-Web a transmis urmatorul raspuns:\n\n{0}", exception.GetBaseException().Message ), this.Text, MessageBoxButtons.OK, MessageBoxIcon.Warning );
        }

        private void OnCompletedSeries( PrescriptionSeriesDownloaderParameters prescriptionDownloaderParameters, string response )
        {
            this.buttonGetSeries.Enabled = true;
            this.textBoxResult.Text = response;
            MessageBox.Show( string.IsNullOrEmpty( response ) ? "Seriile de reteta nu au fost preluate!" : "Seriile de reteta au fost preluata cu success!",
                             this.Text,
                             MessageBoxButtons.OK,
                             string.IsNullOrEmpty( response ) ? MessageBoxIcon.Warning : MessageBoxIcon.Information );
        }
        #endregion
    }
}