﻿#region Copyright ©2007-2013, SIVECO Romania SA - All Rights Reserved
// ======================================================================
// Copyright ©2007-2013 SIVECO Romania SA - All Rights Reserved
// ======================================================================
// This file and its contents are protected by Romanian and International
// copyright laws. Unauthorized reproduction and/or distribution of all
// or any portion of the code contained herein is strictly prohibited
// and will result in severe civil and criminal penalties.
// Any violations of this copyright will be prosecuted
// to the fullest extent possible under law.
// ======================================================================
// THIS COPYRIGHT NOTICE MAY NOT BE REMOVED FROM THIS FILE.
// ======================================================================
#endregion

#region References
using System;
using System.Net;
using System.Security.Cryptography.X509Certificates;
using Cnas.Siui.WebServices.Utils;
#endregion

namespace Cnas.Siui.TestConnectivity.Services.Abstracts
{
    /// <summary>
    /// Basic Abstract Webservice Invoker
    /// </summary>
    /// <typeparam name="TOutput"></typeparam>
    public abstract class AbstractRemoteService<TOutput>
    {
        /// <summary>
        /// Url which the Webservice Method should be invoken on
        /// </summary>
        public string WebServiceUrl { get; private set; }

        /// <summary>
        /// Request timeout
        /// </summary>
        public int RequestTimeout { get; private set; }

        /// <summary>
        /// The certificate that should be attached to the request
        /// </summary>
        public X509Certificate2 Certificate { get; private set; }

        /// <summary>
        /// Constructor
        /// </summary>
        protected AbstractRemoteService( string url, X509Certificate2 certificate, int requestTimeout )
        {
            this.WebServiceUrl = url;
            this.Certificate = certificate;
            this.RequestTimeout = requestTimeout;
        }

        /// <summary>
        /// Invokes the webservice method
        /// </summary>
        public virtual TOutput GetResponse()
        {
            //Creating the request
            var request = this.CreateRequest( new Uri( this.WebServiceUrl ) );

            HttpWebResponse response = null;
            try
            {
                //Retrieving the response
                response = (HttpWebResponse)request.GetResponse();
                //Apply further processing on the web response
                return this.ParseWebResponse( response );
            }
            finally
            {
                if( response != null )
                {
                    response.Close();
                }
            }
        }

        /// <summary>
        /// Parses web response for further information
        /// </summary>
        protected abstract TOutput ParseWebResponse( HttpWebResponse response );

        /// <summary>
        /// Creates and setups the request before calling
        /// </summary>
        protected virtual HttpWebRequest CreateRequest( Uri uri )
        {
            var request = (HttpWebRequest)WebRequest.Create( uri );

            //Making general setups
            RequestHelper.Setup( request );

            if( this.Certificate != null )
            {
                //Attaching the given certificate to the request
                request.ClientCertificates.Add( this.Certificate );
            }
            return request;
        }
    }
}