﻿#region Copyright ©2007-2013, SIVECO Romania SA - All Rights Reserved
// ======================================================================
// Copyright ©2007-2013 SIVECO Romania SA - All Rights Reserved
// ======================================================================
// This file and its contents are protected by Romanian and International
// copyright laws. Unauthorized reproduction and/or distribution of all
// or any portion of the code contained herein is strictly prohibited
// and will result in severe civil and criminal penalties.
// Any violations of this copyright will be prosecuted
// to the fullest extent possible under law.
// ======================================================================
// THIS COPYRIGHT NOTICE MAY NOT BE REMOVED FROM THIS FILE.
// ======================================================================
#endregion

#region References
using System;
using System.Linq;
using System.Net;
using System.Security.Cryptography.X509Certificates;
using Cnas.Siui.TestConnectivity.Services.Abstracts;
using Cnas.Siui.TestConnectivity.Utils;
using Cnas.Siui.WebServices.Utils;
#endregion

namespace Cnas.Siui.TestConnectivity.Services
{
    /// <summary>
    /// Service for retrieving OCSP server token
    /// </summary>
    public class CheckOcspRemoteService : AbstractRemoteService<string>
    {
        /// <summary>
        /// Username required for OCSP authentication
        /// </summary>
        public string Username { get; private set; }

        /// <summary>
        /// Class constructor
        /// </summary>
        public CheckOcspRemoteService( string url, string username, X509Certificate2 certificate, int requestTimeout )
            : base( url, certificate, requestTimeout )
        {
            this.Username = username;
        }

        /// <summary>
        /// Creates the request for retrieving the OCSP server token
        /// </summary>
        protected override HttpWebRequest CreateRequest( Uri uri )
        {
            // setup default uri query params
            Uri uriOcsp = new Uri( string.Format( "{0}?username={1}", uri, this.Username ) );

            string authorityName = this.Certificate.IssuerName.DecodeForSiui();
            if( RequestHelper.IsSpecialAuthority( authorityName ) )
            {
                BigInteger serialNumber = BigInteger.Parse( this.Certificate.SerialNumber, 16U ); // for long serials ulong is not enough
                // Date format: "ddd, dd MMM yyyy HH':'mm':'ss 'GMT'"
                string certNotAfter = this.Certificate.NotAfter.ToUniversalTime().ToString( "yyyy-MM-dd" );
                // setup special uri query params
                uriOcsp = new Uri( uri, string.Format( "?urlParams=true&username={0}&serialNumber={1}&authorityName={2}&clientCertNotAfter={3}", Username, serialNumber, authorityName, certNotAfter ) );
            }

            return base.CreateRequest( uriOcsp );
        }

        /// <summary>
        /// Parses headers response for the OSCP_RESPONSE and retrieves it
        /// </summary>
        protected override string ParseWebResponse( HttpWebResponse response )
        {
            return response != null && response.Headers.Keys.OfType<string>().Contains( "OSCP_RESPONSE" )
                       ? response.Headers["OSCP_RESPONSE"]
                       : null;
        }
    }
}