﻿#region Copyright ©2007-2013, SIVECO Romania SA - All Rights Reserved
// ======================================================================
// Copyright ©2007-2013 SIVECO Romania SA - All Rights Reserved
// ======================================================================
// This file and its contents are protected by Romanian and International
// copyright laws. Unauthorized reproduction and/or distribution of all
// or any portion of the code contained herein is strictly prohibited
// and will result in severe civil and criminal penalties.
// Any violations of this copyright will be prosecuted
// to the fullest extent possible under law.
// ======================================================================
// THIS COPYRIGHT NOTICE MAY NOT BE REMOVED FROM THIS FILE.
// ======================================================================
#endregion

#region References
using System;
using System.IO;
using System.Net;
using System.Security.Cryptography.X509Certificates;
using System.Text;
using System.Xml;
using Cnas.Siui.TestConnectivity.Services.Abstracts;
using Cnas.Siui.WebServices.Utils;
#endregion

namespace Cnas.Siui.TestConnectivity.Services
{
    /// <summary>
    /// Service for testing online connectivity
    /// </summary>
    public class ConnectivityCheckerRemoteService : AbstractOnlineRemoteService<bool>
    {
        public ConnectivityCheckerRemoteService( string url, string ocspUrl, string username, string password, X509Certificate2 certificate, int requestTimeout, bool alwaysRefreshOcspToken = false )
            : base( url, ocspUrl, username, password, certificate, requestTimeout, alwaysRefreshOcspToken ) {}

        /// <summary>
        /// Creates the request to download the wsdl definition
        /// </summary>
        protected override HttpWebRequest CreateRequest( Uri uri )
        {
            //Clearing any cookies
            CookieJar.Clear();
            return base.CreateRequest( new Uri( string.Format( "{0}?wsdl", uri ) ) );
        }

        /// <summary>
        /// Parses the response in search for the wsdl definition. If wsdle definition is found, the return value is true, otherwise false
        /// </summary>
        protected override bool ParseWebResponse( HttpWebResponse response )
        {
            if( response != null )
            {
                using( var retReader = new StreamReader( response.GetResponseStream(), Encoding.UTF8 ) )
                {
                    string xmlContent = retReader.ReadToEnd();

                    try
                    {
                        var xDoc = new XmlDocument();
                        xDoc.LoadXml( xmlContent );
                        // verifies that the downloaded content si correct and returns the result
                        return xDoc.DocumentElement != null && xDoc.DocumentElement.Name == "wsdl:definitions";
                    }
                    catch( Exception ) {}
                }
            }
            //Default value to be returned in case of error or incorrect wsdl definition encountered
            return false;
        }
    }
}