﻿#region Copyright ©2007-2013, SIVECO Romania SA - All Rights Reserved
// ======================================================================
// Copyright ©2007-2013 SIVECO Romania SA - All Rights Reserved
// ======================================================================
// This file and its contents are protected by Romanian and International
// copyright laws. Unauthorized reproduction and/or distribution of all
// or any portion of the code contained herein is strictly prohibited
// and will result in severe civil and criminal penalties.
// Any violations of this copyright will be prosecuted
// to the fullest extent possible under law.
// ======================================================================
// THIS COPYRIGHT NOTICE MAY NOT BE REMOVED FROM THIS FILE.
// ======================================================================
#endregion

#region References
using System;
using System.Linq;
using System.Security.Cryptography.X509Certificates;
using Cnas.Siui.TestConnectivity.Services.Abstracts;
using Cnas.Siui.TestConnectivity.Services.Parameters;
using Cnas.Siui.WebServices.SiuiWS;
#endregion

namespace Cnas.Siui.TestConnectivity.Services
{
    /// <summary>
    /// Service for retrieving the location of the invoked series files. Uses PrescriptionSeriesFileDownloadedRemoteService to download the series xml content 
    /// </summary>
    public class PrescriptionSeriesDownloaderWebService : AbstractWebService<SiuiWSService, string[], string, PrescriptionSeriesDownloaderParameters>
    {
        /// <summary>
        /// Class constructor requires parameters the setup the request.
        /// </summary>
        public PrescriptionSeriesDownloaderWebService( string url, string username, string password, X509Certificate2 certificate, string ocspUrl, int requestTimeout )
            :
                base( url, username, password, certificate, ocspUrl, requestTimeout ) {}

        /// <summary>
        /// Call webservice method and retrieves the url where the series file is located
        /// </summary>
        /// <returns></returns>
        protected override string[] CallWebServiceMethod( SiuiWSService webService, PrescriptionSeriesDownloaderParameters parameters )
        {
            return webService.getPrescriptionSeriesInfo( parameters.CategoryCode, parameters.OrgUnitCode, parameters.UIC, parameters.SubUnitCode );
        }

        /// <summary>
        /// Gets the url for the series file, and passes it to PrescriptionSeriesFileDownloadedRemoteService to download return the xml content.
        /// </summary>
        /// <returns></returns>
        protected override string ProcessResponse( PrescriptionSeriesDownloaderParameters parameters, string[] response )
        {
            if( response == null || response.Length == 0 || string.IsNullOrEmpty( response[0] ) )
            {
                return null;
            }
            var uri = new Uri( this.Url );
            var downloadUri = new Uri( uri, response[0] );
            var service = new PrescriptionSeriesFileDownloadedRemoteService( downloadUri.AbsoluteUri, this.OCSPUrl, this.Username, this.Password, this.Certificate, this.RequestTimeout );
            //Making second request in order to download the series file
            return service.GetResponse();
        }
    }
}