﻿#region Copyright ©2007-2013, SIVECO Romania SA - All Rights Reserved
// ======================================================================
// Copyright ©2007-2013 SIVECO Romania SA - All Rights Reserved
// ======================================================================
// This file and its contents are protected by Romanian and International
// copyright laws. Unauthorized reproduction and/or distribution of all
// or any portion of the code contained herein is strictly prohibited
// and will result in severe civil and criminal penalties.
// Any violations of this copyright will be prosecuted
// to the fullest extent possible under law.
// ======================================================================
// THIS COPYRIGHT NOTICE MAY NOT BE REMOVED FROM THIS FILE.
// ======================================================================
#endregion

#region References
using System;
using System.IO;
using System.Net;
using System.Security.Cryptography.X509Certificates;
using System.Text;
using Cnas.Siui.TestConnectivity.Services.Abstracts;
using ICSharpCode.SharpZipLib.Zip;
#endregion

namespace Cnas.Siui.TestConnectivity.Services
{
    /// <summary>
    /// Service for downloading and unzipping the series file
    /// </summary>
    public class PrescriptionSeriesFileDownloadedRemoteService : AbstractOnlineRemoteService<string>
    {
        /// <summary>
        /// Class constructor requires parameters the setup the request.
        /// </summary>
        public PrescriptionSeriesFileDownloadedRemoteService( string url, string ocspUrl, string username, string password, X509Certificate2 certificate, int requestTimeout )
            : base( url, ocspUrl, username, password, certificate, requestTimeout ) {}

        /// <summary>
        /// Downloads the file series, unzips and returns the xml content
        /// </summary>
        protected override string ParseWebResponse( HttpWebResponse response )
        {
            if( response != null )
            {
                //Creating buffer to read from the response stream
                var buffer = new byte[Environment.Is64BitProcess ? 4096 : 2048];
                using( var stream = response.GetResponseStream() )
                {
                    if( stream != null )
                    {
                        // Stream where to write the unzipped messaje
                        using( var outStream = new MemoryStream() )
                        {
                            using( var zip = new ZipInputStream( stream ) )
                            {
                                if( zip.GetNextEntry() != null )
                                {
                                    int outSize = 0;
                                    // Reading the unzipped bytes
                                    while( ( outSize = zip.Read( buffer, 0, buffer.Length ) ) > 0 )
                                    {
                                        // Writing bytes after unzipping
                                        outStream.Write( buffer, 0, outSize );
                                    }
                                    return Encoding.UTF8.GetString( outStream.ToArray() );
                                }
                            }
                        }
                    }
                }
            }
            return null;
        }
    }
}