﻿#region Copyright ©2007-2013, SIVECO Romania SA - All Rights Reserved
// ======================================================================
// Copyright ©2007-2013 SIVECO Romania SA - All Rights Reserved
// ======================================================================
// This file and its contents are protected by Romanian and International
// copyright laws. Unauthorized reproduction and/or distribution of all
// or any portion of the code contained herein is strictly prohibited
// and will result in severe civil and criminal penalties.
// Any violations of this copyright will be prosecuted
// to the fullest extent possible under law.
// ======================================================================
// THIS COPYRIGHT NOTICE MAY NOT BE REMOVED FROM THIS FILE.
// ======================================================================
#endregion

#region References
using System;
using System.Collections.Generic;
using System.Linq;
using System.Security.Cryptography;
using System.Security.Cryptography.X509Certificates;
#endregion

namespace Cnas.Siui.TestConnectivity.Utils
{
    public static class X509Utils
    {
        /// <summary>
        /// 
        /// </summary>
        /// <returns></returns>
        public static List<X509Certificate2> GetCertificatesList()
        {
            var certList = new X509Certificate2Collection();

            AppendCertificatesFromLocalStore( certList );

            return certList.OfType<X509Certificate2>().ToList();
        }

        private static int AppendCertificatesFromLocalStore( X509Certificate2Collection list )
        {
            int certCount = 0;

            var store = new X509Store( StoreName.My, StoreLocation.CurrentUser );
            store.Open( OpenFlags.ReadOnly | OpenFlags.OpenExistingOnly );

            try
            {
                X509Certificate2Collection collection = store.Certificates;

                var foundDump = new System.Text.StringBuilder( string.Format( "Found {0} certificates in current user's local store:", collection.Count ) );
                foreach( var cert in collection )
                {
                    string keyUsage = ( from X509Extension ext in cert.Extensions
                                        where ext.Oid.FriendlyName == "Key Usage"
                                        select ( (X509KeyUsageExtension)ext ).KeyUsages.ToString() ).FirstOrDefault();

                    var enhancedKeyUsages = ( from X509Extension ext in cert.Extensions
                                              where ext.Oid.FriendlyName == "Enhanced Key Usage"
                                              select ( (X509EnhancedKeyUsageExtension)ext ).EnhancedKeyUsages ).FirstOrDefault();
                    if( enhancedKeyUsages != null )
                    {
                        keyUsage = enhancedKeyUsages.Cast<Oid>().Aggregate( keyUsage, ( current, oid ) => current + ", (X) " + oid.FriendlyName );
                    }

                    foundDump.AppendFormat( "\r\nCertificate #{6}: {{{0}}}\r\n\tSubject: {1}\r\n\tIssued by: {2}\r\n\tValid between {3} and {4}\r\n\tKey usage: {5}",
                                            cert.SerialNumber, cert.Subject, cert.Issuer, cert.NotBefore, cert.NotAfter, keyUsage, ++certCount );
                }

                certCount = 0; // reset counter

                foreach( X509Certificate2 x509 in collection )
                {
                    // clone certificate (don't if it is necessary)
                    var cert = new X509Certificate2();
                    byte[] rawdata = x509.RawData;
                    cert.Import( rawdata );
                    x509.Reset();

                    if( !list.Contains( cert ) )
                    {
                        list.Add( cert );
                        certCount++;
                    }
                }
            }
            catch( CryptographicException ) {}
            finally
            {
                store.Close();
            }
            return certCount;
        }

        /// <summary>
        /// Decodes .NET distinguished name according to RFC2253
        /// </summary>
        /// <param name="issuerName">An X500DistinguishedName object to decode.</param>
        /// <returns>A String representing the decoded value.</returns>
        public static string DecodeForSiui( this X500DistinguishedName issuerName )
        {
            string name = issuerName.Decode( X500DistinguishedNameFlags.UseCommas );
            return name == null ? null : name.ToUpperInvariant().Replace( ", S=", ", ST=" );
        }
    }
}