﻿#region Copyright ©2007-2013, SIVECO Romania SA - All Rights Reserved
// ======================================================================
// Copyright ©2007-2013 SIVECO Romania SA - All Rights Reserved
// ======================================================================
// This file and its contents are protected by Romanian and International
// copyright laws. Unauthorized reproduction and/or distribution of all
// or any portion of the code contained herein is strictly prohibited
// and will result in severe civil and criminal penalties.
// Any violations of this copyright will be prosecuted
// to the fullest extent possible under law.
// ======================================================================
// THIS COPYRIGHT NOTICE MAY NOT BE REMOVED FROM THIS FILE.
// ======================================================================
#endregion

#region References
using System;
using System.Net;
using System.Net.Cache;
#endregion

namespace Cnas.Siui.WebServices.Utils
{
    /// <summary>
    /// Static class used to make general setups for ongoing web requests
    /// </summary>
    public static class RequestHelper
    {
        public static void Setup( HttpWebRequest request )
        {
            // setup service point manager
            ServicePointManager.SecurityProtocol = SecurityProtocolType.Ssl3 | SecurityProtocolType.Tls; // this is the default in .NET
            ServicePointManager.ServerCertificateValidationCallback = ( sender, certificate, chain, errors ) => true; // to disable server certificate validation
            ServicePointManager.Expect100Continue = false; // otherwise we get occasional 505: HTTP version not supported

            // NOTE: see http://stackoverflow.com/questions/1060966/big-files-uploading-webexception-the-connection-was-closed-unexpectedly
            // only small files are transfered, big ones throwing "The connection was closed unexpectedly"
            request.ProtocolVersion = HttpVersion.Version10; // to work with Axis with basic authentication

            // setup request
            request.KeepAlive = false;
            request.AllowAutoRedirect = false;
            request.Proxy = GetSystemWebProxy();
            request.CachePolicy = new RequestCachePolicy( RequestCacheLevel.NoCacheNoStore );

            // set static cookie container
            request.CookieContainer = CookieJar.Cookies;
        }

        public static bool IsSpecialAuthority( string issuerName )
        {
            return Properties.Settings.Default.SpecialCertificationAuthorities.Contains( issuerName );
        }

        private static WebProxy GetSystemWebProxy()
        {
#pragma warning disable 612,618
            WebProxy proxy = WebProxy.GetDefaultProxy();
            proxy.UseDefaultCredentials = true;
            return proxy;
#pragma warning restore 612,618
        }
    }
}